/***************************************************************************/
/**  Archivo    : keypad.c                                                 */
/**  Resumen    : control del keypad del Sistema                           */
/**  Autor		: Miguel Angel Rosales Navarro                             */
/***************************************************************************/
#include "stm32f4xx.h"
#include "stm32f4_discovery.h"
#include "stm32f4xx_gpio.h"
#include <stdio.h>
#include "keypad.h"
#include "rtc.h"
#include "utilidades.h"
#include "display.h"
#include "tm_stm32f4_delay.h"
#include "tm_stm32f4_keypad.h"


void initKeypad(void) {
	// Funcion incluida en "tm_stm32f4_keypad.h"
	TM_KEYPAD_Init(TM_KEYPAD_Type_Large);
}

/** Funcion encargada de la lectura de teclado
 *    devuelve el caracter correspondiente al boton que se esta pulsando
 *    o espacio en blanco si no esta pulsando nada
 */
char readKeypad(void) {
	char returnVal = ' ';
	// Crear instancia de keypad
	// Funcion incluida en "tm_stm32f4_keypad.h"
	TM_KEYPAD_Button_t Keypad_Button = TM_KEYPAD_Read();
	// Si se ha presionado un boton
	if (Keypad_Button != TM_KEYPAD_Button_NOPRESSED) {
		switch (Keypad_Button) {
			case TM_KEYPAD_Button_0:		/* Boton 0 presionado */
				returnVal = '0';
				break;
			case TM_KEYPAD_Button_1:		/* Boton 1 presionado */
				returnVal = '1';
				break;
			case TM_KEYPAD_Button_2:		/* Boton 2 presionado */
				returnVal = '2';
				break;
			case TM_KEYPAD_Button_3:		/* Boton 3 presionado */
				returnVal = '3';
				break;
			case TM_KEYPAD_Button_4:		/* Boton 4 presionado */
				returnVal = '4';
				break;
			case TM_KEYPAD_Button_5:		/* Boton 5 presionado */
				returnVal = '5';
				break;
			case TM_KEYPAD_Button_6:		/* Boton 6 presionado */
				returnVal = '6';
				break;
			case TM_KEYPAD_Button_7:		/* Boton 7 presionado */
				returnVal = '7';
				break;
			case TM_KEYPAD_Button_8:		/* Boton 8 presionado */
				returnVal = '8';
				break;
			case TM_KEYPAD_Button_9:		/* Boton 9 presionado */
				returnVal = '9';
				break;
			case TM_KEYPAD_Button_STAR:		/* Boton * presionado */
				returnVal = '*';
				break;
			case TM_KEYPAD_Button_HASH:		/* Boton # presionado */
				returnVal = '#';
				break;
			case TM_KEYPAD_Button_A:		/* Boton A presionado */
				returnVal = 'A';
				break;
			case TM_KEYPAD_Button_B:		/* Boton B presionado */
				returnVal = 'B';
				break;
			case TM_KEYPAD_Button_C:		/* Boton C presionado */
				returnVal = 'C';
				break;
			case TM_KEYPAD_Button_D:		/* Boton D presionado */
				returnVal = 'D';
				break;
			default:
				returnVal = ' ';
				break;
		}
	}

	return returnVal;
}

/** Funcion encargada de controlar el teclado cuando se ha seleccionado el menu A
 *  (Menu encargado de programar el reloj del Sistema)
 */
void readKeypadMenuA(char character, int *checkingPassword, int *checkingHourCurrent, char *menu, char *previousChar,
		             char* *temporal, int *generalClockConfigured, int *alarmConfigured, int alarmClockConfigured,
		             char* *currentPassword) {
	if ((character == 'A') || (character == 'B') || (character == 'C') || (character == 'D')) {
		*previousChar = ' ';
		*menu = 'A';
	} else if (*checkingPassword == 1) {
		writeDisplay("Introduzca", "Password Actual");
		*temporal = *temporal + character;
		if (length(*temporal) == 4) {
			if (checkPassword(*temporal, *currentPassword) == 1) {
				*checkingHourCurrent = 1;
				writeDisplay("Password", "Correcta");
				Delay(3000);
				writeDisplay("Introduzca", "Hora Actual");
			} else {
				*menu = 'N';
				writeDisplay("Password", "Incorrecta");
				Delay(5000);
				cleanDisplay();
			}
			*temporal = "";
			*checkingPassword = 0;
		}
	} else if (*checkingHourCurrent == 1) {
		writeDisplay("Introduzca", "Hora Actual");
		*temporal = *temporal + character;
		if (length(*temporal) == 4) {
			if (checkHour(*temporal) == 1) {
				programRTC(temporal);
				*generalClockConfigured = 1;
				if (alarmClockConfigured == 1) {
					*alarmConfigured = 1;
				} else {
					*alarmConfigured = 0;
				}
				writeDisplay("Hora Actual", "Configurada");
			} else {
				writeDisplay("Hora Actual", "Incorrecta");
			}
			*checkingHourCurrent = 0;
			*menu = 'N';
			*temporal = "";
			Delay(5000);
			cleanDisplay();
		}
	}
}

/** Funcion encargada de controlar el teclado cuando se ha seleccionado el menu B
 *  (Menu encargado de cambiar el password del Sistema)
 */
void readKeypadMenuB(char character, int *checkingPassword, int *checkingPasswordNew, int *checkingPasswordNewAux,
		                 char *menu, char *previousChar, char* *temporal, char* *temporalAux, char* *currentPassword) {
	if ((character == 'A') || (character == 'B') || (character == 'C') || (character == 'D')) {
		*previousChar = ' ';
		*menu = 'B';
	} else if (*checkingPassword == 1) {
		writeDisplay("Introduzca", "Password Actual");
		*temporal = *temporal + character;
		if (length(*temporal) == 4) {
			if (checkPassword(*temporal, *currentPassword) == 1) {
				*checkingPasswordNew = 1;
				writeDisplay("Password", "Correcta");
				Delay(3000);
				writeDisplay("Introduzca", "Nueva Password");
			} else {
				*menu = 'N';
				writeDisplay("Password", "Incorrecta");
				Delay(5000);
				cleanDisplay();
			}
			*checkingPassword = 0;
			*temporal = "";
		}
	} else if (*checkingPasswordNew == 1) {
		writeDisplay("Introduzca", "Nueva Password");
		*temporal = *temporal + character;
		if (length(*temporal) == 4) {
			*checkingPasswordNew = 0;
			*checkingPasswordNewAux = 1;
			writeDisplay("Confirme", "Nueva Password");
		}
	} else if (*checkingPassword == 1) {
		writeDisplay("Confirme", "Nueva Password");
		*temporalAux = *temporalAux + character;
		if (length(*temporalAux) == 4) {
			if (checkPassword(*temporal, *temporalAux) == 1) {
				*currentPassword = *temporal;
				writeDisplay("Cambio de Password", "Correcto");
			} else {
				writeDisplay("Cambio Password", "Incorrecto");
			}
			*checkingPasswordNewAux = 0;
			*temporal = "";
			*temporalAux = "";
			*menu = 'N';
			Delay(5000);
			cleanDisplay();
		}
	}
}

/** Funcion encargada de controlar el teclado cuando se ha seleccionado el menu C
 *  (Menu encargado de programar las horas de inicio y fin de alarma activa del Sistema)
 */
void readKeypadMenuC(char character, int *checkingPassword, int *checkingHourBegin, int *checkingHourEnd, char *menu, char *previousChar,
		                 char* *temporal, char* *temporalAux, int *currentAlarmClockBegin, int *currentAlarmClockEnd,
		                 int generalClockConfigured, int *alarmConfigured, int *alarmClockConfigured, char* *currentPassword) {
	if ((character == 'A') || (character == 'B') || (character == 'C') || (character == 'D')) {
		*previousChar = ' ';
		*menu = 'C';
	} else if (*checkingPassword == 1) {
		writeDisplay("Introduzca", "Password Actual");
		*temporal = *temporal + character;
		if (length(*temporal) == 4) {
			if (checkPassword(*temporal, *currentPassword) == 1) {
				*checkingHourBegin = 1;
				writeDisplay("Password", "Correcta");
				Delay(3000);
				writeDisplay("Introduzca", "Hora Inicio");
			} else {
				*menu = 'N';
				writeDisplay("Password", "Incorrecta");
				Delay(5000);
				cleanDisplay();
			}
			*checkingPassword = 0;
			*temporal = "";
		}
	} else if (*checkingHourBegin == 1) {
		writeDisplay("Introduzca", "Hora Inicio");
		*temporal = *temporal + character;
		if (length(*temporal) == 4) {
			if (checkHour(*temporal) == 1) {
				*checkingHourEnd = 1;
				writeDisplay("Hora Inicio", "Correcta");
				Delay(3000);
				writeDisplay("Introduzca", "Hora Fin");
			} else {
				writeDisplay("Hora Inicio", "Incorrecta");
				*menu = 'N';
				Delay(5000);
				cleanDisplay();
			}
			*checkingHourBegin = 0;
			*temporal = "";
		}
	} else if (*checkingHourEnd == 1) {
		writeDisplay("Introduzca", "Hora Fin");
		*temporalAux = *temporalAux + character;
		if (length(*temporalAux) == 4) {
			if (checkHour(*temporalAux) == 1) {
				*currentAlarmClockBegin = (atoi(temporal[0])*1000) + (atoi(temporal[1])*100)
						                  + (atoi(temporal[2])*10) + atoi(temporal[3]);
				*currentAlarmClockEnd = (atoi(temporalAux[0])*1000) + (atoi(temporalAux[1])*100)
						                + (atoi(temporalAux[2])*10) + atoi(temporalAux[3]);
				*alarmClockConfigured = 1;
				if (generalClockConfigured == 1) {
					*alarmConfigured = 1;
				} else {
					*alarmConfigured = 0;
				}
				writeDisplay("Hora Fin", "Correcta");
				Delay(3000);
				writeDisplay("Alarma", "Programada");
			} else {
				writeDisplay("Hora Fin", "Incorrecta");
			}
			*checkingHourEnd = 0;
			*temporal = "";
			*temporalAux = "";
			*menu = 'N';
			Delay(5000);
			cleanDisplay();
		}
	}
}

/** Funcion encargada de controlar el teclado cuando se ha seleccionado el menu D
 *  (Menu encargado de programar iniciar o parar la alarma del Sistema)
 */
void readKeypadMenuD(char character, int *checkingPassword, char *menu, char *previousChar, char* *temporal,
		             int *alarmConfigured, int *alarmStarted, int *alarmFired, char* *currentPassword) {
	if ((character == 'A') || (character == 'B') || (character == 'C') || (character == 'D')) {
		*previousChar = ' ';
		*menu = 'D';
	} else if (*checkingPassword == 1) {
		writeDisplay("Introduzca", "Password Actual");
		*temporal = *temporal + character;
		if (length(*temporal) == 4) {
			if (checkPassword(*temporal, *currentPassword) == 1) {
				writeDisplay("Password", "Correcta");
				Delay(3000);
				if (*alarmConfigured == 1) {
					if ((*alarmFired == 1) || (*alarmStarted == 1)) {
						writeDisplay("Alarma", "Parada");
						stopAlarm(alarmFired, alarmStarted);
					} else {
						writeDisplay("Alarma", "Iniciada");
						startAlarm(alarmStarted);
					}
				} else {
					writeDisplay("Configure", "Alarma");
				}
			} else {
				writeDisplay("Password", "Incorrecta");
			}
			*menu = 'N';
			*temporal = "";
			*checkingPassword = 0;
			Delay(5000);
			cleanDisplay();
		}
	}

}

void controlKeypad(int *generalClockConfigured, int *alarmClockConfigured, int *alarmConfigured, int *alarmStarted,
		            int *alarmFired, char *menu, char* *temporal, char* *temporalAux, char *previousChar,
		            int *currentAlarmClockBegin, int *currentAlarmClockEnd, char* *currentPassword,
		            int *checkingPassword, int *checkingPasswordNew, int *checkingPasswordNewAux, int *checkingHourCurrent,
		            int *checkingHourBegin, int *checkingHourEnd) {

	char character = readKeypad();
	if ((character != ' ') && (character != '*') && (character != '#')) {
		if (*menu == 'N') {
			if (character == 'A') {
				readKeypadMenuA(character, checkingPassword, checkingHourCurrent, menu, previousChar, temporal,
						        generalClockConfigured, alarmConfigured, *alarmClockConfigured, currentPassword);
			} else if (character == 'B') {
				readKeypadMenuB(character, checkingPassword, checkingPasswordNew, checkingPasswordNewAux,
						        menu, previousChar, temporal, temporalAux, currentPassword);
			} else if (character == 'C') {
				readKeypadMenuC(character, checkingPassword, checkingHourBegin, checkingHourEnd, menu, previousChar,
						        temporal, temporalAux, currentAlarmClockBegin, currentAlarmClockEnd,
						        *generalClockConfigured, alarmConfigured, alarmClockConfigured, currentPassword);
			} else if (character == 'D') {
				readKeypadMenuD(character, checkingPassword, menu, previousChar, temporal,
						        alarmConfigured, alarmStarted, alarmFired, currentPassword);
			}
		} else {
			*checkingPassword = 1;
			if (*menu == 'A') {
				readKeypadMenuA(character, checkingPassword, checkingHourCurrent, menu, previousChar, temporal,
						generalClockConfigured, alarmConfigured, *alarmClockConfigured, currentPassword);
			} else if (*menu == 'B') {
				readKeypadMenuB(character, checkingPassword, checkingPasswordNew, checkingPasswordNewAux,
						        menu, previousChar, temporal, temporalAux, currentPassword);
			} else if (*menu == 'C') {
				readKeypadMenuC(character, checkingPassword, checkingHourBegin, checkingHourEnd, menu, previousChar,
								temporal, temporalAux, currentAlarmClockBegin, currentAlarmClockEnd,
								*generalClockConfigured, alarmConfigured, alarmClockConfigured, currentPassword);
			} else if (*menu == 'D') {
				readKeypadMenuD(character, checkingPassword, menu, previousChar, temporal,
						        alarmConfigured, alarmStarted, alarmFired, currentPassword);
			} else {
				*previousChar = ' ';
			}
		}
	} else {
		*previousChar = ' ';
	}
}
/***************************************************************************/

