/***************************************************************************/
/**  Archivo    : leds.c                                                   */
/**  Resumen    : control de los leds del Sistema                          */
/**  Autor		: Miguel Angel Rosales Navarro                             */
/***************************************************************************/
#include "stm32f4xx.h"
#include "stm32f4_discovery.h"
#include "stm32f4xx_gpio.h"
#include <stdio.h>
#include "leds.h"
#include "tm_stm32f4_delay.h"
#include "defines.h"

/** Constante para referencia del led rojo */
int LED_RED = 0;
/** Constante para referencia del led naranja */
int LED_ORANGE = 1;
/** Constante para referencia del led verde */
int LED_GREEN = 2;

void initLeds(void) {
	// LED ROJO
	GPIO_InitTypeDef GPIO_InitDefRed;
	RCC_AHB1PeriphClockCmd(CLK_LINE_LED_RED, ENABLE);
	GPIO_InitDefRed.GPIO_Pin = PIN_LED_RED;
	GPIO_InitDefRed.GPIO_Mode = GPIO_Mode_OUT;
	GPIO_InitDefRed.GPIO_OType = GPIO_OType_PP;
	GPIO_InitDefRed.GPIO_PuPd = GPIO_PuPd_UP;
	GPIO_InitDefRed.GPIO_Speed = GPIO_Speed_100MHz;
	GPIO_Init(GPIO_LED_RED, &GPIO_InitDefRed);
	GPIO_ResetBits(GPIO_LED_RED, PIN_LED_RED);
	// LED NARANJA
	GPIO_InitTypeDef GPIO_InitDefOrange;
	RCC_AHB1PeriphClockCmd(CLK_LINE_LED_ORANGE, ENABLE);
	GPIO_InitDefOrange.GPIO_Pin = PIN_LED_ORANGE;
	GPIO_InitDefOrange.GPIO_Mode = GPIO_Mode_OUT;
	GPIO_InitDefOrange.GPIO_OType = GPIO_OType_PP;
	GPIO_InitDefOrange.GPIO_PuPd = GPIO_PuPd_UP;
	GPIO_InitDefOrange.GPIO_Speed = GPIO_Speed_100MHz;
	GPIO_Init(GPIO_LED_ORANGE, &GPIO_InitDefOrange);
	GPIO_ResetBits(GPIO_LED_ORANGE, PIN_LED_ORANGE);
	// LED VERDE
	GPIO_InitTypeDef GPIO_InitDefGreen;
	RCC_AHB1PeriphClockCmd(CLK_LINE_LED_GREEN, ENABLE);
	GPIO_InitDefGreen.GPIO_Pin = PIN_LED_GREEN;
	GPIO_InitDefGreen.GPIO_Mode = GPIO_Mode_OUT;
	GPIO_InitDefGreen.GPIO_OType = GPIO_OType_PP;
	GPIO_InitDefGreen.GPIO_PuPd = GPIO_PuPd_UP;
	GPIO_InitDefGreen.GPIO_Speed = GPIO_Speed_100MHz;
	GPIO_Init(GPIO_LED_GREEN, &GPIO_InitDefGreen);
	GPIO_ResetBits(GPIO_LED_GREEN, PIN_LED_GREEN);
}

void activateLed(int activateLed, int numLed) {
	if (numLed == LED_RED) {
		if (numLed == 1) {
			GPIO_SetBits(GPIO_LED_RED, PIN_LED_RED);
		} else {
			GPIO_ResetBits(GPIO_LED_RED, PIN_LED_RED);
		}
	} else if (numLed == LED_ORANGE) {
		if (numLed == 1) {
			GPIO_SetBits(GPIO_LED_ORANGE, PIN_LED_ORANGE);
		} else {
			GPIO_ResetBits(GPIO_LED_ORANGE, PIN_LED_ORANGE);
		}
	} else if (numLed == LED_GREEN) {
		if (numLed == 1) {
			GPIO_SetBits(GPIO_LED_GREEN, PIN_LED_GREEN);
		} else {
			GPIO_ResetBits(GPIO_LED_GREEN, PIN_LED_GREEN);
		}
	}
}

/** Funcion que hace parpadear un led (seleccionado con constantes de led */
void blinkingLed(int numLed) {
	activateLed(numLed,1);
	Delay(100);
	activateLed(numLed,0);
	Delay(100);
	activateLed(numLed,1);
	Delay(100);
	activateLed(numLed,0);
}

void controlLeds(int alarmFired, int alarmStarted, int alarmConfigured) {
	if (alarmFired == 1) {
		blinkingLed(LED_RED);
		activateLed(LED_GREEN, 1);
		activateLed(LED_ORANGE, 0);
	} else if (alarmStarted == 1) {
		activateLed(LED_GREEN, 1);
		activateLed(LED_ORANGE, 0);
		activateLed(LED_RED, 0);
	} else if (alarmConfigured == 1) {
		activateLed(LED_ORANGE, 1);
		activateLed(LED_GREEN, 0);
		activateLed(LED_RED, 0);
	} else {
		activateLed(LED_RED, 1);
		activateLed(LED_GREEN, 0);
		activateLed(LED_ORANGE, 0);
	}
}
/***************************************************************************/
