/***************************************************************************/
/**  Archivo    : utilidades.d                                             */
/**  Resumen    : libreria con funciones utiles a usar por el Sistema      */
/**  Autor		: Miguel Angel Rosales Navarro                             */
/***************************************************************************/
#include "stm32f4xx.h"
#include "stm32f4_discovery.h"

#ifndef _UTILIDADES_H_
#define _UTILIDADES_H_

/** Devuelve la longitud de la cadena argumento */
int length(char* arrayChar);

/** Comprueba que las dos cadenas argumento son iguales:
 *    Devuelve: 0 = no son iguales; 1 = son iguales
 */
int checkPassword(char* password, char* auxPassword);

/** Comprueba que la cadena argumento contiene una hora valida: numero entre 0 y 2359
 *    Devuelve: 1 = la hora es valida; 0 = la hora no es valida
 */
int checkHour(char* hour);

/** Comprueba si una hora se encuentra entre otras dos horas:
 *    currentTime: hora a comprobar
 *    currentAlarmClockBegin: hora inicio del intervalo a comprobar
 *    currentAlarmClockEnd: hora fin del intervalo a comprobar
 *    Devuelve: 1 = la hora esta en el intervalo; 0 = la hora no esta en el intervalo
 */
int alarmInProgrammedTime(int currentTime, int currentAlarmClockBegin, int currentAlarmClockEnd);

/** Dispara la alarma del Sistema actualizando la variable de estado alarmFired */
void fireAlarm(int *alarmFired);

/** Para la alarma del Sistema actualizando las variables de estado alarmFired y alarmStarted */
void stopAlarm(int *alarmFired, int *alarmStarted);

/** Inicia la alarma del Sistema actualizando la variable de estado alarmStarted */
void startAlarm(int *alarmStarted);

/** Funcion principal para controlar los sensores (movimiento y magnetico) del Sistema
 *  y en funcion de las variables de estado disparar la alarma
 */
void controlSensors(int *alarmFired, int alarmStarted, int currentAlarmClockBegin, int currentAlarmClockEnd);

#endif
/***************************************************************************/
